@@ -6,30 +6,16 @@
 ###          on input, y and dy contains the initial values of the state 
 ###          variables and rates of changes for times[1]
 ###          parms is a vector of parameters for func.  They should not
-###          change during the integration. `rtol', and `atol'
-###          are, respectively, the relative tolerance parameter, and the
-###          absolute tolerance parameter.  `atol' may be scaler or vector.
-###          `rtol' is a scalar or a vector.
-###
-###          The return value is a matrix whose rows correspond to the values
-###          in `times', and columns to the elements of `y'.
-###
-###          'res' may be a string instead of an R function.  If
-###          so, then if jacres is not NULL, it must be a character string
-###          as well.  In these cases, 'res' is the name
-###          of a function to be found in the dll named 'dllname' 
-###          (without extension). 'jacres' points to the name of the Jacobian.
+###          change during the integration.
 ### ============================================================================
 
-
-
-
-daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
+daspk   <- function(y, times, func=NULL, parms, dy=NULL, res=NULL,
     nalg=0, rtol=1e-6, atol=1e-8, jacfunc=NULL, jacres=NULL,
     jactype = "fullint", estini = NULL, verbose=FALSE, tcrit = NULL,
     hmin=0, hmax=NULL, hini=0, ynames=TRUE, maxord =5, bandup=NULL,
     banddown=NULL, maxsteps=5000, dllname=NULL, initfunc=dllname,
-    initpar=parms, rpar=NULL, ipar=NULL,nout=0, outnames=NULL, ...) {
+    initpar=parms, rpar=NULL, ipar=NULL,nout=0, outnames=NULL,
+    forcings=NULL, initforc = NULL, fcontrol=NULL, ...) {
 
 ### check input 
   if (!is.numeric(y))
@@ -60,9 +46,9 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
   if (!is.null(jacres) && !(is.function(jacres) || is.character(jacres)))
     stop("`jacres' must be a function or character vector")
   if (length(atol) > 1 && length(atol) != n)
-    stop("`atol' must either be a scaler, or as long as `y'")
+    stop("`atol' must either be a scalar, or as long as `y'")
   if (length(rtol) > 1 && length(rtol) != n)
-    stop("`rtol' must either be a scaler, or as long as `y'")
+    stop("`rtol' must either be a scalar, or as long as `y'")
   if (!is.numeric(hmin))
     stop("`hmin' must be numeric")
   if (hmin < 0)
@@ -84,23 +70,23 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
 
 ### Jacobian, method flag
   if (jactype == "fullint" )
-    imp <- 22 # full Jacobian, calculated internally
+    imp <- 22 # full, calculated internally
   else if (jactype == "fullusr" )
-    imp <- 21 # full Jacobian, specified by user function
+    imp <- 21 # full, specified by user function
   else if (jactype == "bandusr" )
-    imp <- 24 # banded Jacobian, specified by user function
+    imp <- 24 # banded, specified by user function
   else if (jactype == "bandint" )
-    imp <- 25 # banded Jacobian, specified internally
-  else stop("jactype must be one of fullint, fullusr, bandusr or bandint")
+    imp <- 25 # banded, calculated internally
+  else stop("'jactype' must be one of 'fullint', 'fullusr', 'bandusr' or 'bandint'")
 
   if (imp %in% c(24,25) && is.null(bandup))
-    stop("daspk: bandup must be specified if banded Jacobian")
+    stop("'bandup' must be specified if banded Jacobian")
   if (imp %in% c(24,25) && is.null(banddown))
-    stop("daspk: banddown must be specified if banded Jacobian")
+    stop("'banddown' must be specified if banded Jacobian")
 
-##  if (miter == 4) Jacobian should have empty banddown empty rows-vode+daspk only! 
+  #  if (miter == 4) Jacobian should have banddown empty rows-vode+daspk only!
   if (imp == 24)
-    erow<-matrix(nc=n,nr=banddown,0)
+    erow<-matrix(data=0,ncol=n,nrow=banddown)
   else erow<-NULL
     
   if (is.null(banddown))
@@ -121,6 +107,7 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
   PsolFunc<- NULL
     
   ModelInit <- NULL
+  flist<-list(fmat=0,tmat=0,imat=0,ModelForc=NULL)
 
   if (!is.null(dllname))  {
     if (is.loaded(initfunc, PACKAGE = dllname, type = "") ||
@@ -128,6 +115,8 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
       ModelInit <- getNativeSymbolInfo(initfunc, PACKAGE = dllname)$address
     } else if (initfunc != dllname && ! is.null(initfunc))
        stop(paste("cannot integrate: initfunc not loaded ",initfunc))
+    if (! is.null(forcings))
+      flist <- checkforcings(forcings,times,dllname,initforc,verbose,fcontrol)
   }
 
   ## If res is a character vector, then
@@ -196,11 +185,11 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
     if (is.null(initfunc))
       initpar <- NULL # parameter initialisation not needed if function is not a DLL
     
-    rho <- environment(func)
     ## func or res and jac are overruled, either including ynames, or not
     ## This allows to pass the "..." arguments and the parameters
 
     if (is.null(res))  {               # res is NOT specified, func is
+      rho <- environment(func)
       Res    <- function(time,y,dy) {
         if (ynames) attr(y,"names")  <- Ynames
         FF <-func   (time,y,parms,...)
@@ -212,6 +201,7 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
          func   (time,y,parms,...)
       }
     } else {                       # res is specified
+      rho <- environment(res)
       Res   <- function(time,y,dy){
         if (ynames) {
           attr(y,"names")  <- Ynames
@@ -231,7 +221,7 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
     ## the Jacobian
     if (! is.null(jacfunc)) {        # Jacobian associated with func
 
-      tmp <- eval(jacfunc(times[1], y, dy,parms, 1, ...), rho)
+      tmp <- eval(jacfunc(times[1], y, parms, ...), rho)
       if (! is.matrix(tmp))
         stop("jacfunc must return a matrix\n")
 
@@ -240,14 +230,16 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
           attr(y,"names")  <- Ynames
           attr(dy,"names") <- dYnames
         }
-        JF <- -1* rbind(jacfunc(Rin[1],y,dy,parms,...),erow)
-        if (imp %in% c(24,25))
+        JF <- -1* jacfunc(Rin[1],y,parms,...)
+        if (imp %in% c(24,25)) {
           JF[bandup+1,]<-JF[bandup+1,]+Rin[2]
+          JF <- rbind(erow,JF )
+          }
         else
-          JF           <-JF + diag(nc=n,x=Rin[2])
+          JF           <-JF + diag(nc=n,nr=n,x=Rin[2])
         return(JF)
       }
-    } else if (! is.null(jacres)) { # Jacobian associated with res
+    } else if (! is.null(jacres)) { # Jacobian given
        tmp <- eval(jacres(times[1], y, dy, parms, 1, ...), rho)
        if (! is.matrix(tmp))
          stop("jacres must return a matrix\n")
@@ -261,7 +253,7 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
            attr(y,"names")  <- Ynames
            attr(dy,"names") <- dYnames
          }
-         rbind(jacres(Rin[1],y,dy,parms,Rin[2],...),erow)
+         rbind(erow,jacres(Rin[1],y,dy,parms,Rin[2],...))
        }
     } else JacRes <- NULL
          
@@ -332,7 +324,7 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
 
   info[9] <- maxord!=5
  
-  if (! is.null (estini)) info[11] <- estini        # daspk will estimate dy and algebraic equ.
+  if (! is.null (estini)) info[11] <- estini # daspk will estimate dy and algebraic equ.
   if (info[11] > 2 || info[11]< 0 ) stop("daspk: illegal value for estini")
     
 # length of rwork and iwork 
@@ -393,12 +385,15 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
       JacRes, ModelInit, PsolFunc, as.integer(verbose),as.integer(info),
       as.integer(iwork),as.double(rwork), as.integer(Nglobal),as.integer(maxIt),
       as.integer(bandup),as.integer(banddown),as.integer(nrowpd),
-      as.double (rpar), as.integer(ipar),PACKAGE = "deSolve")
+      as.double (rpar), as.integer(ipar), flist,
+      PACKAGE = "deSolve")
 
 ### saving results    
 
   out [1,1] <- times[1]
   istate <- attr(out, "istate")
+  istate <- setIstate(istate,iin=c(1,8:9,12:20),
+                      iout=c(1,6,5,2:4,13,12,19,9,8,11))
   rstate <- attr(out, "rstate")
 
   ## ordinary output variables already estimated
@@ -409,7 +404,8 @@ daspk          <- function(y, times, func=NULL, parms,  dy=NULL,  res=NULL,
   attr(out, "istate") <- istate
   attr(out, "rstate") <- rstate
   attr(out, "type") <- "daspk"
+  class(out) <- c("deSolve","matrix")    # a differential equation
   dimnames(out) <- list(nm, NULL)
   if (verbose) diagnostics(out)
-  return(t(out))
+  t(out)
 }
