@@ -2,7 +2,8 @@
 #include <string.h>
 #include "deSolve.h"
 
-/* definition of the calls to the fortran functions - in file opkdmain.f*/
+/* definition of the calls to the FORTRAN functions - in file opkdmain.f
+and in file dvode.f**/
 
 void F77_NAME(dlsoda)(void (*)(int *, double *, double *, double *, double *, int *),
 		     int *, double *, double *, double *,
@@ -37,12 +38,31 @@ void F77_NAME(dlsodar)(void (*)(int *, double *, double *, double *, double *, i
 		     void (*)(int *, double *, double *, int *, double *),  /* rootfunc */
          int *, int *, double *, int *);
 
-/* interface between fortran function call and R function 
+void F77_NAME(dvode)(void (*)(int *, double *, double *, double *,
+                              double *, int *),
+		     int *, double *, double *, double *,
+		     int *, double *, double *, int *, int *,
+		     int *, double *,int *,int *, int *,
+		     void (*)(int *, double *, double *, int *,
+			            int *, double *, int *, double*, int*),
+		     int *, double *, int *);
+
+/* KS: wrapper above the derivate function that first estimates the
+values of the forcing functions */
+
+static void forc_lsoda (int *neq, double *t, double *y,
+                         double *ydot, double *yout, int *iout)
+{
+  updatedeforc(t);
+  derfun(neq, t, y, ydot, yout, iout);
+}
+
+/* interface between FORTRAN function call and R function
    Fortran code calls lsoda_derivs(N, t, y, ydot, yout, iout) 
    R code called as odesolve_deriv_func(time, y) and returns ydot 
    Note: passing of parameter values and "..." is done in R-function lsodx*/
 
-static void lsoda_derivs (int *neq, double *t, double *y, 
+static void lsoda_derivs (int *neq, double *t, double *y,
                           double *ydot, double *yout, int *iout)
 {
   int i;
@@ -60,7 +80,7 @@ static void lsoda_derivs (int *neq, double *t, double *y,
 }
 
 /* only if lsodar: 
-   interface between fortran call to root and corresponding R function */
+   interface between FORTRAN call to root and corresponding R function */
 
 static void lsoda_root (int *neq, double *t, double *y, int *ng, double *gout)
 {
@@ -77,7 +97,7 @@ static void lsoda_root (int *neq, double *t, double *y, int *ng, double *gout)
   my_unprotect(2);
 }
 
-/* interface between fortran call to jacobian and R function */
+/* interface between FORTRAN call to jacobian and R function */
 
 static void lsoda_jac (int *neq, double *t, double *y, int *ml,
 		    int *mu, double *pd, int *nrowpd, double *yout, int *iout)
@@ -97,7 +117,7 @@ static void lsoda_jac (int *neq, double *t, double *y, int *ml,
 }
 
 /* only if lsodes: 
-   interface between fortran call to jacvec and corresponding R function */
+   interface between FORTRAN call to jacvec and corresponding R function */
 
 static void lsoda_jacvec (int *neq, double *t, double *y, int *j,
 		    int *ian, int *jan, double *pdj, double *yout, int *iout)
@@ -119,15 +139,11 @@ static void lsoda_jacvec (int *neq, double *t, double *y, int *j,
 
 
 /* give name to data types */
-typedef void deriv_func(int *, double *, double *,double *, double *, int *);
 typedef void root_func (int *, double *, double *,int *, double *);
 typedef void jac_func  (int *, double *, double *, int *,
 		                    int *, double *, int *, double *, int *);
 typedef void jac_vec   (int *, double *, double *, int *,
 		                    int *, int *, double *, double *, int *);
-typedef void init_func (void (*)(int *, double *));
-
-
 
 /* MAIN C-FUNCTION, CALLED FROM R-code */
 
@@ -135,7 +151,7 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 		SEXP atol, SEXP rho, SEXP tcrit, SEXP jacfunc, SEXP initfunc,
 		SEXP verbose, SEXP iTask, SEXP rWork, SEXP iWork, SEXP jT, 
     SEXP nOut, SEXP lRw, SEXP lIw, SEXP Solver, SEXP rootfunc, 
-    SEXP nRoot, SEXP Rpar, SEXP Ipar, SEXP Type)
+    SEXP nRoot, SEXP Rpar, SEXP Ipar, SEXP Type, SEXP flist)
 
 {
 /******************************************************************************/
@@ -145,18 +161,17 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 /* These R-structures will be allocated and returned to R*/
   SEXP yout, yout2=NULL, ISTATE, RWORK, IROOT=NULL;    
 
-  int  i, j, k, l, m, ll, ij, nt, repcount, latol, lrtol, lrw, liw, isOut, maxit, solver;
-  double *xytmp, *rwork, tin, tout, *Atol, *Rtol, *out, *dy=NULL, ss;
-  int neq, itol, itask, istate, iopt, *iwork, jt, mflag, nout, ntot, is;
-  int nroot, *jroot=NULL, isroot, *ipar, lrpar, lipar, isDll;
-  int type, nspec, nx, ny, nz, Nt, bndx, bndy, isp;
-  
+  int  i, j, k, nt, repcount, latol, lrtol, lrw, liw;
+  int  maxit, solver, isForcing;
+  double *xytmp, *rwork, tin, tout, *Atol, *Rtol, *dy=NULL, ss;
+  int neq, itol, itask, istate, iopt, *iwork, jt, mflag,  is;
+  int nroot, *jroot=NULL, isroot,  isDll, type;
+
   deriv_func *derivs;
   jac_func   *jac=NULL;
   jac_vec    *jacvec;
   root_func  *root=NULL;
-  init_func  *initializer;
-    
+
 /******************************************************************************/
 /******                         STATEMENTS                               ******/
 /******************************************************************************/
@@ -172,51 +187,19 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
   maxit = 10;                   /* number of iterations */ 
   mflag = INTEGER(verbose)[0];
  
-  nout   = INTEGER(nOut)[0];    /* number of output variables */
   nroot  = INTEGER(nRoot)[0];   /* number of roots (lsodar) */
-  solver = INTEGER(Solver)[0];  /* 1= lsoda, 2=lsode: 3=lsodeS, 4=lsodar */
+  solver = INTEGER(Solver)[0];  /* 1=lsoda,2=lsode,3=lsodeS,4=lsodar,5=vode */
 
-/* The output:
-    out and ipar are used to pass output variables (number set by nout)
-    followed by other input (e.g. forcing functions) provided 
-    by R-arguments rpar, ipar
-    ipar[0]: number of output variables, ipar[1]: length of rpar, 
-    ipar[2]: length of ipar */
-  
-  if (inherits(func, "NativeSymbol"))  /* function is a dll */
-  {
+/* is function a dll ?*/
+  if (inherits(func, "NativeSymbol")) {
    isDll = 1;
-   if (nout > 0) isOut = 1; 
-   ntot  = neq + nout;          /* length of yout */
-   lrpar = nout + LENGTH(Rpar); /* length of rpar; LENGTH(Rpar) is always >0 */
-   lipar = 3 + LENGTH(Ipar);    /* length of ipar */
-
-  } else                              /* function is not a dll */
-  {
+  } else {
    isDll = 0;
-   isOut = 0;
-   ntot = neq;
-   lipar = 1;
-   lrpar = 1; 
   }
- 
-   out   = (double *) R_alloc(lrpar, sizeof(double));
-   ipar  = (int *)    R_alloc(lipar, sizeof(int));
 
-   if (isDll ==1)
-   {
-    ipar[0] = nout;              /* first 3 elements of ipar are special */
-    ipar[1] = lrpar;
-    ipar[2] = lipar;
-    /* other elements of ipar are set in R-function lsodx via argument *ipar* */
-    for (j = 0; j < LENGTH(Ipar);j++) ipar[j+3] = INTEGER(Ipar)[j];
+/* initialise output ... */
+  initOut(isDll, neq, nOut, Rpar, Ipar);
 
-    /* first nout elements of rpar reserved for output variables 
-      other elements are set in R-function lsodx via argument *rpar* */
-    for (j = 0; j < nout; j++)        out[j] = 0.;  
-    for (j = 0; j < LENGTH(Rpar);j++) out[nout+j] = REAL(Rpar)[j];
-   }
-   
 /* copies of all variables that will be changed in the FORTRAN subroutine */
 
   xytmp = (double *) R_alloc(neq, sizeof(double));
@@ -241,141 +224,52 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 
   if (solver ==3)
   {
-  type   = INTEGER(Type)[0];    
-  if (type == 2)           /* 1-D problem ; Type contains further information */
-    {
-    nspec = INTEGER(Type)[1]; /* number of components*/ 
-    nx    = INTEGER(Type)[2]; /* dimension x*/ 
-
-       ij     = 31+neq;
-       iwork[30] = 1;
-       k = 1;
-       for( i = 0; i<nspec; i++) {
-         for( j = 0; j<nx; j++) {       
-          if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
-                       iwork[ij++] = k;
-           if (j<nx-1) iwork[ij++] = k+1 ;
-           if (j>0)    iwork[ij++] = k-1 ;
-
-            for(l = 0; l<nspec;l++) 
-              if (l != i) iwork[ij++] = l*nx+j+1;
-
-            iwork[30+k] = ij-30-neq;
-            k=k+1;  
-          }
-        }
-       iwork[ij] = 0; 
-                                                
-}
-     else if (type == 3) { /* 2-D problem */
-       nspec = INTEGER(Type)[1]; /* number components*/ 
-       nx    = INTEGER(Type)[2]; /* dimension x*/ 
-       ny    = INTEGER(Type)[3]; /* dimension y*/
-       bndx  = INTEGER(Type)[4]; /* cyclic boundary x*/
-       bndy  = INTEGER(Type)[5]; /* cyclic boundary y*/
-       Nt    = nx*ny;
-       ij     = 31+neq;
-       iwork[30] = 1;
-       m = 1; 
-       for( i = 0; i<nspec; i++) {
-         isp = i*Nt;
-         for( j = 0; j<nx; j++) {       
-           for( k = 0; k<ny; k++) {       
-           if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
-                                iwork[ij++] = m;
-             if (k<ny-1)        iwork[ij++] = m+1;
-
-             if (j<nx-1)        iwork[ij++] = m+ny;
-             if (j >0)          iwork[ij++] = m-ny;
-             if (k >0)          iwork[ij++] = m-1;
-             if (bndx == 1) {
-               if (j == 0)      iwork[ij++] = isp+(nx-1)*ny+k+1;
-               if (j == nx-1)   iwork[ij++] = isp+k+1;
-             }
-             if (bndy == 1) {
-               if (k == 0)      iwork[ij++] = isp+(j+1)*ny;
-               if (k == ny-1)   iwork[ij++] = isp + j*ny +1;
-             }
-            for(l = 0; l<nspec;l++)
-              if (l != i)       iwork[ij++] = l*Nt+j*ny+k+1;
-       
-            iwork[30+m] = ij-30-neq;
-            m = m+1;
-            }
-          }
-        }   
-       }
-     else if (type == 4) { /* 3-D problem */
-       nspec = INTEGER(Type)[1]; /* number components*/
-       nx    = INTEGER(Type)[2]; /* dimension x*/
-       ny    = INTEGER(Type)[3]; /* dimension y*/
-       nz    = INTEGER(Type)[4]; /* dimension y*/
-/*       bndx  = INTEGER(Type)[5];
-       bndy  = INTEGER(Type)[6];  cyclic boundary NOT yet implemented*/
-       Nt    = nx*ny*nz;
-       ij     = 31+neq;
-       iwork[30] = 1;
-       m = 1;
-       for( i = 0; i<nspec; i++) {
-         isp = i*Nt;
-         for( j = 0; j<nx; j++) {
-           for( k = 0; k<ny; k++) {
-             for( ll = 0; ll<nz; ll++) {
-              if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
-                                 iwork[ij++] = m;
-              if (ll<nz-1)       iwork[ij++] = m+1;
-              if (k<ny-1)        iwork[ij++] = m+nz;
-              if (j<nx-1)        iwork[ij++] = m+ny*nz;
-
-              if (j >0)          iwork[ij++] = m-ny*nz;
-              if (k >0)          iwork[ij++] = m-nz;
-              if (ll >0)         iwork[ij++] = m-1;
-              for(l = 0; l<nspec;l++)
-                if (l != i)       iwork[ij++] = l*Nt+j*ny*nz+k*nz+ll+1;
-
-              iwork[30+m] = ij-30-neq;
-              m = m+1;
-              }
-            }
-          }
-        }
-       }    }
+    type   = INTEGER(Type)[0];
+    if (type == 2)        /* 1-D problem ; Type contains further information */
+       sparsity1D( Type, iwork, neq, liw) ;
+    else if (type == 3)  /* 2-D problem */
+       sparsity2D( Type, iwork, neq, liw);
+    else if (type == 4)  /* 3-D problem */
+     sparsity3D (Type, iwork, neq, liw);
+  }
 
 /* initialise global R-variables... */
 
   PROTECT(Time = NEW_NUMERIC(1));                  incr_N_Protect();
   PROTECT(Y = allocVector(REALSXP,(neq)));         incr_N_Protect();
   PROTECT(yout = allocMatrix(REALSXP,ntot+1,nt));  incr_N_Protect();
-  PROTECT(de_gparms = parms);                      incr_N_Protect();
 
- /* The initialisation routine */
-  if (!isNull(initfunc))
-	  {
-	  initializer = (init_func *) R_ExternalPtrAddr(initfunc);
-	  initializer(Initdeparms);
-	  }
+  /**************************************************************************/
+  /****** Initialization of Parameters and Forcings (DLL functions)    ******/
+  /**************************************************************************/
+  initParms(initfunc, parms);
+  isForcing = initForcings(flist);
 
 /* pointers to functions derivs, jac, jacvec and root, passed to FORTRAN */
 
-  if (isDll ==1) 
-    { /* DLL address passed to fortran */
+  if (isDll) 
+    { /* DLL address passed to FORTRAN */
       derivs = (deriv_func *) R_ExternalPtrAddr(func);  
       /* no need to communicate with R - but output variables set here */
       if (isOut) {dy = (double *) R_alloc(neq, sizeof(double));
                   for (j = 0; j < neq; j++) dy[j] = 0.; }
 	  
+	  /* here overruling derivs if forcing */
+      if (isForcing) {
+        derfun = (deriv_func *) R_ExternalPtrAddr(func);
+        derivs = (deriv_func *) forc_lsoda;
+      }
     } else {
-      /* interface function between fortran and R passed to Fortran*/ 
+      /* interface function between FORTRAN and R passed to FORTRAN */
       derivs = (deriv_func *) lsoda_derivs; 
       /* needed to communicate with R */
       odesolve_deriv_func = func;
       odesolve_envir = rho;
-
     }
 
   if (!isNull(jacfunc) && solver !=3)  /* lsodes uses jacvec */
     {
-      if (isDll ==1)
+      if (isDll)
 	    {
 	     jac = (jac_func *) R_ExternalPtrAddr(jacfunc);
 	    } else  {
@@ -384,9 +278,9 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 	    }
     }
 
-  if (!isNull(jacfunc) && solver ==3)
+  if (!isNull(jacfunc) && solver ==3)   /*lsodes*/
     {
-      if (isDll ==1)
+      if (isDll)
 	    {
 	     jacvec = (jac_vec *) R_ExternalPtrAddr(jacfunc);
 	    } else  {
@@ -395,11 +289,11 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 	    }
     }
 
-  if (solver == 4 && nroot > 0)      /* lsodar */
+  if (solver == 4 && nroot > 0)        /* lsodar */
   { jroot = (int *) R_alloc(nroot, sizeof(int));
      for (j=0; j<nroot; j++) jroot[j] = 0;
   
-    if (isDll ==1) 
+    if (isDll) 
     {
       root = (root_func *) R_ExternalPtrAddr(rootfunc);
     } else {
@@ -414,6 +308,9 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
   if (latol == 1 && lrtol  > 1 ) itol = 3;
   if (latol  > 1 && lrtol  > 1 ) itol = 4;
 
+  for (j = 0; j < lrtol; j++) Rtol[j] = REAL(rtol)[j];
+  for (j = 0; j < latol; j++) Atol[j] = REAL(atol)[j];
+
   itask = INTEGER(iTask)[0];   
   istate = 1;
 
@@ -436,98 +333,95 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
                   }
 
 /*                     ####   main time loop   ####                           */    
-  for (i = 0; i < nt-1; i++)
-    {
-      tin = REAL(times)[i];
-      tout = REAL(times)[i+1];
-      repcount = 0;
-      for (j = 0; j < lrtol; j++) Rtol[j] = REAL(rtol)[j];
-      for (j = 0; j < latol; j++) Atol[j] = REAL(atol)[j];
-      do
+  for (i = 0; i < nt-1; i++) {
+    tin = REAL(times)[i];
+    tout = REAL(times)[i+1];
+    repcount = 0;
+    do
 	{  /* error control */
-	  if (istate == -2)
-	    {
+ 	    if (istate == -2) {
 	      for (j = 0; j < lrtol; j++) Rtol[j] *= 10.0;
 	      for (j = 0; j < latol; j++) Atol[j] *= 10.0;
 	      warning("Excessive precision requested.  `rtol' and `atol' have been scaled upwards by the factor %g\n",10.0);
 	      istate = 3;
 	    }
 
-    if (solver == 1)
-    {	    
-	  F77_CALL(dlsoda) (derivs, &neq, xytmp, &tin, &tout,
-			   &itol, NUMERIC_POINTER(rtol), NUMERIC_POINTER(atol), 
-         &itask, &istate, &iopt, rwork,
+      if (solver == 1) {
+	      F77_CALL(dlsoda) (derivs, &neq, xytmp, &tin, &tout,
+			   &itol, Rtol, Atol, &itask, &istate, &iopt, rwork,
 			   &lrw, iwork, &liw, jac, &jt, out, ipar); 
-    } else if (solver == 2) {
-    F77_CALL(dlsode) (derivs, &neq, xytmp, &tin, &tout,
-			   &itol, NUMERIC_POINTER(rtol), NUMERIC_POINTER(atol), 
-         &itask, &istate, &iopt, rwork,
+      } else if (solver == 2) {
+        F77_CALL(dlsode) (derivs, &neq, xytmp, &tin, &tout,
+			   &itol, Rtol, Atol, &itask, &istate, &iopt, rwork,
 			   &lrw, iwork, &liw, jac, &jt, out, ipar); 
-    } else if (solver == 3) {
-    F77_CALL(dlsodes) (derivs, &neq, xytmp, &tin, &tout,
-			   &itol, NUMERIC_POINTER(rtol), NUMERIC_POINTER(atol), 
-         &itask, &istate, &iopt, rwork,
+      } else if (solver == 3) {
+        F77_CALL(dlsodes) (derivs, &neq, xytmp, &tin, &tout,
+			   &itol, Rtol, Atol, &itask, &istate, &iopt, rwork,
 			   &lrw, iwork, &liw, jacvec, &jt, out, ipar); 
-    } else if (solver == 4) {
-    F77_CALL(dlsodar) (derivs, &neq, xytmp, &tin, &tout,
-			   &itol, NUMERIC_POINTER(rtol), NUMERIC_POINTER(atol), 
-         &itask, &istate, &iopt, rwork,
+      } else if (solver == 4) {
+        F77_CALL(dlsodar) (derivs, &neq, xytmp, &tin, &tout,
+			   &itol, Rtol, Atol,  &itask, &istate, &iopt, rwork,
 			   &lrw, iwork, &liw, jac, &jt, root, &nroot, jroot, 
          out, ipar); 
-    }
+      } else if (solver == 5) {
+ 	      F77_CALL(dvode) (derivs, &neq, xytmp, &tin, &tout,
+			   &itol, Rtol, Atol, &itask, &istate, &iopt, rwork,
+			   &lrw, iwork, &liw, jac, &jt, out, ipar);
+      }
     
-	  if (istate == -1) 
-     {
-      warning("an excessive amount of work (> maxsteps ) was done, but integration was not successful - increase maxsteps");
-     }
-	  if (istate == 3 && solver == 4)
-	    { istate = -20;  repcount = 50;  
+	    if (istate == -1)  {
+        warning("an excessive amount of work (> maxsteps ) was done, but integration was not successful - increase maxsteps");
+      } else if (istate == 3 && solver == 4){
+	      istate = -20;  repcount = 50;
+      } else if (istate == -2)  {
+	      warning("Excessive precision requested.  scale up `rtol' and `atol' e.g by the factor %g\n",10.0);
+	    } else if (istate == -4)  {
+        warning("repeated error test failures on a step, but integration was successful - singularity ?");
+      } else if (istate == -5)  {
+        warning("repeated convergence test failures on a step, but integration was successful - inaccurate Jacobian matrix?");
+      } else if (istate == -6)  {
+        warning("Error term became zero for some i: pure relative error control (ATOL(i)=0.0) for a variable which is now vanished");
       }
 
-	  repcount ++;
+	    repcount ++;
 	} while (tin < tout && istate >= 0 && repcount < maxit); 
 	
-  if (istate == -3)
-	{
+  if (istate == -3)  {
+    error("illegal input detected before taking any integration steps - see written message");
 	  unprotect_all();
-	  error("Illegal input to lsoda\n");
-	}
-  else
-	{
+	}  else	{
 	  REAL(yout)[(i+1)*(ntot+1)] = tin;
 	  for (j = 0; j < neq; j++)
 	    REAL(yout)[(i+1)*(ntot + 1) + j + 1] = xytmp[j];
-	  if (isOut == 1) 
-    {
-        derivs (&neq, &tin, xytmp, dy, out, ipar) ;
-	      for (j = 0; j < nout; j++)
-	       REAL(yout)[(i+1)*(ntot + 1) + j + neq + 1] = out[j]; 
+	  if (isOut == 1)  {
+      derivs (&neq, &tin, xytmp, dy, out, ipar) ;
+	    for (j = 0; j < nout; j++)
+	      REAL(yout)[(i+1)*(ntot + 1) + j + neq + 1] = out[j];
     }
 	}
 	  
 /*                    ####  an error occurred   ####                          */    
-  if (istate < 0 || tin < tout) {
-	  if (istate != -20) warning("Returning early.  Results are accurate, as far as they go\n");
+   if (istate < 0 || tin < tout) {
+	  if (istate != -20) warning("Returning early. Results are accurate, as far as they go\n");
 
 	 /* need to redimension yout here, and add the attribute "istate" for */
 	 /* the most recent value of `istate' from lsodx */
 	  PROTECT(yout2 = allocMatrix(REALSXP,ntot+1,(i+2)));incr_N_Protect();
-	  for (k = 0; k < i+2; k++)
-	   for (j = 0; j < ntot+1; j++)
+    for (k = 0; k < i+2; k++)
+	    for (j = 0; j < ntot+1; j++)
 	     REAL(yout2)[k*(ntot+1) + j] = REAL(yout)[k*(ntot+1) + j];
-	  break;
-      }
-    }     /* end main time loop */
+	    break;
+    }
+  }     /* end main time loop */
 
 /*                   ####   returning output   ####                           */    
-  if (istate == -20 && nroot > 0) 
-   { isroot = 1   ;
-     PROTECT(IROOT = allocVector(INTSXP, nroot));incr_N_Protect();
-     for (k = 0;k<nroot;k++) INTEGER(IROOT)[k] = jroot[k];
-   } else isroot = 0;
+  if (istate == -20 && nroot > 0)  {
+    isroot = 1   ;
+    PROTECT(IROOT = allocVector(INTSXP, nroot));incr_N_Protect();
+    for (k = 0;k<nroot;k++) INTEGER(IROOT)[k] = jroot[k];
+  } else isroot = 0;
 
-  PROTECT(ISTATE = allocVector(INTSXP, 22));incr_N_Protect();
+  PROTECT(ISTATE = allocVector(INTSXP, 23));incr_N_Protect();
   for (k = 0;k<22;k++) INTEGER(ISTATE)[k+1] = iwork[k];
         
   PROTECT(RWORK = allocVector(REALSXP, 5));incr_N_Protect();
@@ -535,18 +429,16 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 
   INTEGER(ISTATE)[0] = istate;  
   if (istate == -20) INTEGER(ISTATE)[0] = 3; 	  
-  if (istate > 0)
-    {
-      setAttrib(yout, install("istate"), ISTATE);
-      setAttrib(yout, install("rstate"), RWORK);    
-      if (isroot==1) setAttrib(yout, install("iroot"), IROOT);    
-      }
-  else
-    {
-      setAttrib(yout2, install("istate"), ISTATE);
-      setAttrib(yout2, install("rstate"), RWORK);    
-      if (isroot==1) setAttrib(yout2, install("iroot"), IROOT);    
-      }
+  if (istate > 0) {
+    setAttrib(yout, install("istate"), ISTATE);
+    setAttrib(yout, install("rstate"), RWORK);
+    if (isroot==1) setAttrib(yout, install("iroot"), IROOT);
+  }
+  else  {
+    setAttrib(yout2, install("istate"), ISTATE);
+    setAttrib(yout2, install("rstate"), RWORK);
+    if (isroot==1) setAttrib(yout2, install("iroot"), IROOT);
+  }
 
 /*                       ####   termination   ####                            */    
   unprotect_all();
